@@ -184,25 +184,25 @@ setClass("PictureDefinitions",
          prototype(content = list()))
 
 setGeneric("getDef",
-           function(object, id) standardGeneric("getDef"))
+           function(id) standardGeneric("getDef"))
 
 setMethod("getDef",
-          signature(object = "PictureDefinitions",
-                    id = "character"),
-          function(object, id) {
-              object@content[[id]]
+          signature(id = "character"),
+          function(id) {
+              defList <- get("defs", .grImport2Env)
+              defList@content[[id]]
           })
 
 setGeneric("setDef",
-           function(object, id, value) standardGeneric("setDef"))
+           function(id, value) standardGeneric("setDef"))
 
 setMethod("setDef",
-          signature(object = "PictureDefinitions",
-                    id = "character",
+          signature(id = "character",
                     value = "PictureContent"),
-          function(object, id, value) {
-              object@content[[id]] <- value
-              object
+          function(id, value) {
+              defList <- get("defs", .grImport2Env)
+              defList@content[[id]] <- value
+              assign("defs", defList, .grImport2Env)
           })
 
 setClass("PictureSummary",
@@ -271,17 +271,30 @@ setMethod("[[", "PictureGroup",
 # NOTE that if the name does not identify a known definition then the
 # function silently returns the old name
 transformRegisteredDef <- function(id, tm) {
-    defList <- get("defs", .grImport2Env)
-    def <- getDef(defList, id)
+    def <- getDef(id)
     if (! is.null(def)) {
         newDef <- applyTransform(def, tm)
         newId <- updateId(id)
-        assign("defs", setDef(defList, newId, newDef), .grImport2Env)
+        setDef(newId, newDef)
         id <- newId
     }
     id
 }
 
+## Different again because need to transform the clip path itself
+## and the registered definition 
+transformClipPath <- function(cp, tm) {
+    id <- cp@label
+    def <- getDef(id)
+    if (! is.null(def)) {
+        newDef <- applyTransform(def, tm)
+        newId <- updateId(id)
+        setDef(newId, newDef)
+        cp@label <- newId
+    }
+    applyTransform(cp, tm)
+}
+
 setMethod("applyTransform",
           signature(object = "gpar",
                     tm = "matrix"),
@@ -383,6 +396,29 @@ setMethod("applyTransform",
           function(object, tm) {
               object@content <- lapply(object@content, applyTransform, tm)
               object@gp <- applyTransform(object@gp, tm)
+              ## If there is a clip path or a mask, also transform those
+              if (!is.null(object@clip)) {
+                  object@clip <- transformClipPath(object@clip, tm)
+              }
+              if (length(object@maskRef)) {  ## !(NULL or character(0))
+                  object@maskRef <- transformRegisteredDef(object@maskRef, tm)
+              }
+              object
+          })
+              
+setMethod("applyTransform",
+          signature(object = "PictureClipPath",
+                    tm = "matrix"),
+          function(object, tm) {
+              object@content <- lapply(object@content, applyTransform, tm)
+              object
+          })
+              
+setMethod("applyTransform",
+          signature(object = "PictureMask",
+                    tm = "matrix"),
+          function(object, tm) {
+              object@content <- lapply(object@content, applyTransform, tm)
               object
           })
               
@@ -402,19 +438,38 @@ setMethod("applyTransform",
 transformRect <- function(object, tm) {
     # Get new x, y, width, height, and angle from
     # original x, y, width, height, and transform
-    loc <- tm %*% c(object@x, object@y, 1)
-    sx <- sqrt(tm[1, 1]^2 + tm[1, 2]^2)
-    if (tm[1, 1] < 0)
-        sx <- -sx
-    sy <- sqrt(tm[2, 1]^2 + tm[2, 2]^2)
-    if (tm[2, 2] < 0)
-        sy <- -sy
-    angle <- atan2(-tm[1, 2], tm[1, 1])
-    object@x <- loc[1, 1]
-    object@y <- loc[2, 1]
-    object@width <- sx*object@width
-    object@height <- sy*object@height
-    object@angle <- object@angle + angle
+    ## https://math.stackexchange.com/questions/13150/extracting-rotation-scale-values-from-2d-transformation-matrix
+    a <- tm[1, 1]
+    b <- tm[1, 2]
+    c <- tm[2, 1]
+    d <- tm[2, 2]
+    e <- tm[1, 3]
+    f <- tm[2, 3]    
+    delta <- a * d - b * c
+    translation <- c(e, f)
+    if (a != 0 || b != 0) {
+        r <- sqrt(a * a + b * b)
+        rotation <- if (b > 0) acos(a / r) else -acos(a / r)
+        scale <- c(r, delta / r)
+        skew <- c(atan((a * c + b * d) / (r * r)), 0)
+    } else if (c != 0 || d != 0) {
+        s <- sqrt(c * c + d * d)
+        rotation <- pi / 2 -
+            if (d > 0) acos(-c / s) else -acos(c / s)
+        scale <- c(delta / s, s)
+        skew <- c(0, atan((a * c + b * d) / (s * s)))
+    } else {
+        ## a = b = c = d = 0
+        warning("Zero transformation matrix")
+    }
+    if (any(skew != 0)) {
+        warning("Skew transformation matrix")
+    }
+    object@x <- object@x + translation[1]
+    object@y <- object@y + translation[2]
+    object@width <- scale[1]*object@width
+    object@height <- scale[2]*object@height
+    object@angle <- object@angle + rotation
     object
 }
 
@@ -460,7 +515,7 @@ transformRectBBox <- function(object, tm) {
                    ncol = 3)
     for (i in seq_len(nrow(locs)))
         locs[i, ] <- tm %*% locs[i, ]
-    c(min(locs[, 1]), min(locs[, 2]), max(locs[, 1]), max(locs[, 2]))
+    c(min(locs[, 1]), max(locs[, 1]), min(locs[, 2]), max(locs[, 2]))
 }
 
 setMethod("applyTransform",
@@ -483,9 +538,36 @@ setMethod("applyTransform",
               # (which modifies object's x,y,width,height)
               object@bbox <- transformRectBBox(object, tm)
               object <- transformRect(object, tm)
+              ## If there is a mask, also transform that
+              if (length(object@maskRef)) {  ## !(NULL or character(0))
+                  object@maskRef <- transformRegisteredDef(object@maskRef, tm)
+              }
               object 
           })
 
+setMethod("applyTransform",
+          signature(object = "Picture",
+                    tm = "matrix"),
+          function(object, tm) {
+              content <- lapply(object@content, applyTransform, tm)
+              defs <- new("PictureDefinitions",
+                          content = lapply(object@defs@content,
+                                           applyTransform, tm))
+              allBounds <- lapply(content, getbbox)
+              xmin <- min(sapply(allBounds, function(x) x[1]))
+              xmax <- max(sapply(allBounds, function(x) x[2]))
+              ymin <- min(sapply(allBounds, function(x) x[3]))
+              ymax <- max(sapply(allBounds, function(x) x[4]))
+              bbox <- c(xmin, xmax, ymin, ymax)
+              summary <- new("PictureSummary",
+                             xscale = bbox[1:2],
+                             yscale = rev(bbox[3:4]))
+              new("Picture",
+                  content = content,
+                  defs = defs,
+                  summary = summary)
+          })
+
 getbboxFromList <- function(x) {
     allBounds <- lapply(x, getbbox)
     xmin <- min(sapply(allBounds, function(x) x[1]))
